import re
from typing import Match

from pelican import signals
from pelican.contents import Content
from pelican.utils import slugify

SETTINGS_NAME = "WIKILINKS"


def replace_wikilinks(text: str) -> str:
    def repl(group: Match):
        href, text = group.groups()
        if not text:
            text = href
        href = slugify(
            href,
            regex_subs=[
                (
                    r"[^\w\s-]",
                    "",
                ),  # remove non-alphabetical/whitespace/'-' chars
                (r"(?u)\A\s*", ""),  # strip leading whitespace
                (r"(?u)\s*\Z", ""),  # strip trailing whitespace
                (
                    r"[-\s]+",
                    "-",
                ),  # reduce multiple whitespace or '-' to single '-'
            ],
        )
        return f'<a href="/{href}/">{text}</a>'

    if text:
        return re.sub(r"\[\[([^|\]]+)\|?(.*?)\]\]", repl, text)


def content_object_init(instance: Content):
    instance._content = replace_wikilinks(instance._content)


def register():
    signals.content_object_init.connect(content_object_init)



from . import tasks
from invoke import Collection, task


__ALL__ = ["site"]
TASKS = Collection("tasks")


def add_task(t, **project_args):
    @task(name=t.name, optional=t.optional)
    def wrapped_task(c, **task_args):
        return t(c, **project_args, **task_args)

    wrapped_task.__doc__ = t.__doc__
    TASKS.add_task(wrapped_task, name=t.__name__)


def site(host="localhost", port=8000, domain=None):
    add_task(tasks.build)
    add_task(tasks.livereload, host=host, port=port)
    add_task(tasks.publish, domain=domain)
    return TASKS
